﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Markdig;
using Markdig.Extensions.Yaml;
using Markdig.Syntax;
using YamlDotNet.Serialization;
using YamlDotNet.Serialization.NamingConventions;

namespace git_fault
{
    public class HtmlRenderer : IRenderer
    {
        public HtmlRenderer()
        {
        }

        public void NewFaultCreated(string sha, string fault)
        {
            Console.WriteLine($"<p>Issue with Id: {sha} was created.</p>");
        }

        public void FaultUpdated(string id, string oldText, string newText)
        {
            Console.WriteLine($"<p>Issue with Id: {id} was updated.</p>");
        }

        public void FaultDeleted(string id, string oldText)
        {
            Console.WriteLine($"<p>Issue with Id: {id} was deleted.</p>");
        }

        public void FaultsListed(bool full, List<(string, string, List<(string, string)>)> faults)
        {
            var deserializer = new DeserializerBuilder()
                .WithNamingConvention(PascalCaseNamingConvention.Instance)
                .IgnoreUnmatchedProperties()
                .Build();

            MarkdownPipeline Pipeline = new MarkdownPipelineBuilder()
                .UseYamlFrontMatter()
                .Build();


            foreach (var fault in faults)
            {
                var document = Markdown.Parse(fault.Item2, Pipeline);
                var block = document.Descendants<YamlFrontMatterBlock>().FirstOrDefault();
                var input = new StringReader(fault.Item2);
                var yaml = block.Lines
                    .Lines
                    .OrderByDescending(x => x.Line)
                    .Select(x => $"{x}\n")
                    .ToList()
                    .Select(x => x.Replace("---", string.Empty))
                    .Select(x => x.Replace("...", string.Empty))
                    .Where(x => !string.IsNullOrWhiteSpace(x))
                    .Aggregate((s, agg) => agg + s);
                var faultmeta = deserializer.Deserialize<FaultMeta>(yaml);
                Console.WriteLine("<ul>");
                Console.WriteLine($"<li>Id: {fault.Item1}</li>");
                Console.WriteLine($"<li>Priority: {faultmeta.Priority}</li>");
                Console.WriteLine($"<li>Title: {faultmeta.Title}</li>");
                Console.WriteLine($"<li>Status: {faultmeta.Status}</li>");
                Console.WriteLine("</ul>");

                if (full)
                {
                    var faulthtml = Markdown.ToHtml(fault.Item2, new MarkdownPipelineBuilder().UseYamlFrontMatter().Build());
                    Console.WriteLine(faulthtml);
                }

                Console.WriteLine("");
                Console.WriteLine("<h2>Appendix</h2>");
                Console.WriteLine("<ul>");
                foreach (var appendix in fault.Item3)
                {
                    Console.WriteLine($"<li>Id: {appendix.Item1}</li>");
                    Console.WriteLine("<li>" + appendix.Item2 + "</li>");
                }

                Console.WriteLine("</ul>");
                Console.WriteLine("");
            }
        }

        public void ShowFault(string fault)
        {
            var deserializer = new DeserializerBuilder()
                .WithNamingConvention(PascalCaseNamingConvention.Instance)
                .IgnoreUnmatchedProperties()
                .Build();

            MarkdownPipeline Pipeline = new MarkdownPipelineBuilder()
                .UseYamlFrontMatter()
                .Build();

            var document = Markdown.Parse(fault, Pipeline);
            var block = document.Descendants<YamlFrontMatterBlock>().FirstOrDefault();
            var input = new StringReader(fault);
            var yaml = block.Lines
                .Lines
                .OrderByDescending(x => x.Line)
                .Select(x => $"{x}\n")
                .ToList()
                .Select(x => x.Replace("---", string.Empty))
                .Select(x => x.Replace("...", string.Empty))
                .Where(x => !string.IsNullOrWhiteSpace(x))
                .Aggregate((s, agg) => agg + s);
            var faultmeta = deserializer.Deserialize<FaultMeta>(yaml);

            Console.WriteLine("<ul>");
            Console.WriteLine($"<li>Priority: {faultmeta.Priority}</li>");
            Console.WriteLine($"<li>Title: {faultmeta.Title}</li>");
            Console.WriteLine($"<li>Status: {faultmeta.Status}</li>");
            Console.WriteLine("</ul>");
            var faulthtml = Markdown.ToHtml(fault, new MarkdownPipelineBuilder().UseYamlFrontMatter().Build());
            Console.WriteLine(faulthtml);
        }

        public void ShowAppendix(string appendix, string name)
        {
            Console.WriteLine($"<p>Appendix: {name}</p>");
            Console.WriteLine("<div>");
            Console.WriteLine(appendix);
            Console.WriteLine("</div>");
        }

        public void Error(string msg)
        {
            Console.WriteLine(msg);
        }

        public class FaultMeta
        {
            public string Status { get; set; }
            public string Title { get; set; }
            public string Priority { get; set; }
        }
    }
}