﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Markdig;
using Markdig.Extensions.Yaml;
using Markdig.Syntax;
using YamlDotNet.Serialization;
using YamlDotNet.Serialization.NamingConventions;

namespace git_fault
{
    public class TextRenderer : IRenderer
    {

        public TextRenderer()
        {
        }
        public void NewFaultCreated(string sha, string fault)
        {
            Console.WriteLine($"Issue with Id: {sha} was created.");
        }

        public void FaultUpdated(string id, string oldText, string newText)
        {
            Console.WriteLine($"Issue with Id: {id} was updated.");
        }

        public void FaultDeleted(string id, string oldText)
        {
            Console.WriteLine($"Issue with Id: {id} was deleted.");
        }

        public void FaultsListed(bool full, List<(string, string, List<(string, string)>)> faults)
        {
            var deserializer = new DeserializerBuilder()
               .WithNamingConvention(PascalCaseNamingConvention.Instance)
               .IgnoreUnmatchedProperties()
               .Build();

            MarkdownPipeline Pipeline = new MarkdownPipelineBuilder()
                .UseYamlFrontMatter()
                .Build();


            foreach (var fault in faults)
            {
                var document = Markdown.Parse(fault.Item2, Pipeline);
                var block = document.Descendants<YamlFrontMatterBlock>().FirstOrDefault();
                var input = new StringReader(fault.Item2);
                var yaml = block.Lines
                                .Lines
                                .OrderByDescending(x => x.Line)
                                .Select(x => $"{x}\n")
                                .ToList()
                                .Select(x => x.Replace("---", string.Empty))
                                .Select(x => x.Replace("...", string.Empty))
                                .Where(x => !string.IsNullOrWhiteSpace(x))
                                .Aggregate((s, agg) => agg + s);
                var faultmeta = deserializer.Deserialize<FaultMeta>(yaml);
                Console.WriteLine($"Id: {fault.Item1}");
                Console.WriteLine($"Priority: {faultmeta.Priority}");
                Console.WriteLine($"Title: {faultmeta.Title}");
                Console.WriteLine($"Status: {faultmeta.Status}");

                if (full)
                {
                    var faulthtml = Markdown.ToPlainText(fault.Item2, new MarkdownPipelineBuilder().UseYamlFrontMatter().Build());
                    Console.WriteLine(faulthtml);
                }
                Console.WriteLine("");
                Console.WriteLine("Appendix:");
                foreach (var appendix in fault.Item3)
                {
                    Console.WriteLine($"Id: {appendix.Item1}");
                    Console.WriteLine($"Name: {appendix.Item2}");
                }
                Console.WriteLine("");
            }
        }

        public void ShowFault(string fault)
        {
            var deserializer = new DeserializerBuilder()
              .WithNamingConvention(PascalCaseNamingConvention.Instance)
              .IgnoreUnmatchedProperties()
              .Build();

            MarkdownPipeline Pipeline = new MarkdownPipelineBuilder()
                .UseYamlFrontMatter()
                .Build();

            var document = Markdown.Parse(fault, Pipeline);
            var block = document.Descendants<YamlFrontMatterBlock>().FirstOrDefault();
            var input = new StringReader(fault);
            var yaml = block.Lines
                            .Lines
                            .OrderByDescending(x => x.Line)
                            .Select(x => $"{x}\n")
                            .ToList()
                            .Select(x => x.Replace("---", string.Empty))
                            .Select(x => x.Replace("...", string.Empty))
                            .Where(x => !string.IsNullOrWhiteSpace(x))
                            .Aggregate((s, agg) => agg + s);
            var faultmeta = deserializer.Deserialize<FaultMeta>(yaml);

            Console.WriteLine($"Priority: {faultmeta.Priority}");
            Console.WriteLine($"Title: {faultmeta.Title}");
            Console.WriteLine($"Status: {faultmeta.Status}");
            var faulthtml = Markdown.ToPlainText(fault, new MarkdownPipelineBuilder().UseYamlFrontMatter().Build());
            Console.WriteLine(faulthtml);
        }

        public void ShowAppendix(string appendix, string name)
        {
            Console.WriteLine($"Appendix: {name}");
            Console.WriteLine("");
            Console.WriteLine(appendix);
        }
        
        public void Error(string msg)
        {
            Console.WriteLine(msg);
        }

        public class FaultMeta
        {
            public string Status { get; set; }
            public string Title { get; set; }
            public string Priority { get; set; }
        }
    }
}
